/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2016 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "myParabolicVelocityFvPatchVectorField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"

// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //



// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::myParabolicVelocityFvPatchVectorField::
myParabolicVelocityFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchVectorField(p, iF),
    maxValue_(0.0),
    n_(Zero),
    y_(Zero)
{
}


Foam::myParabolicVelocityFvPatchVectorField::
myParabolicVelocityFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchVectorField(p, iF),
    maxValue_(readScalar(dict.lookup("maxValue"))),
    n_(pTraits<vector>(dict.lookup("n"))),
    y_(pTraits<vector>(dict.lookup("y")))
{

    if (mag(n_) < SMALL || mag(y_) < SMALL)
    {
         FatalErrorIn("myParabolicVelocityFvPatchVectorField(dict)")
             << "n or y given with zero size not correct"
             << abort(FatalError);
    }
 
    n_ /= mag(n_);
    y_ /= mag(y_);

    fixedValueFvPatchVectorField::evaluate();

    /*
    //Initialise with the value entry if evaluation is not possible
    fvPatchVectorField::operator=
    (
        vectorField("value", dict, p.size())
    );
    */
}


Foam::myParabolicVelocityFvPatchVectorField::
myParabolicVelocityFvPatchVectorField
(
    const myParabolicVelocityFvPatchVectorField& ptf,
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchVectorField(ptf, p, iF, mapper),
    maxValue_(ptf.maxValue_),
    n_(ptf.n_),
    y_(ptf.y_)
{}


Foam::myParabolicVelocityFvPatchVectorField::
myParabolicVelocityFvPatchVectorField
(
    const myParabolicVelocityFvPatchVectorField& ptf
)
:
    fixedValueFvPatchVectorField(ptf),
    maxValue_(ptf.maxValue_),
    n_(ptf.n_),
    y_(ptf.y_)
{}


Foam::myParabolicVelocityFvPatchVectorField::
myParabolicVelocityFvPatchVectorField
(
    const myParabolicVelocityFvPatchVectorField& ptf,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchVectorField(ptf, iF),
    maxValue_(ptf.maxValue_),
    n_(ptf.n_),
    y_(ptf.y_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::myParabolicVelocityFvPatchVectorField::autoMap
(
    const fvPatchFieldMapper& m
)
{
    fixedValueFvPatchVectorField::autoMap(m);

}


void Foam::myParabolicVelocityFvPatchVectorField::rmap
(
    const fvPatchVectorField& ptf,
    const labelList& addr
)
{
    fixedValueFvPatchVectorField::rmap(ptf, addr);

    const myParabolicVelocityFvPatchVectorField& tiptf =
        refCast<const myParabolicVelocityFvPatchVectorField>(ptf);


}


void Foam::myParabolicVelocityFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }
    boundBox bb(patch().patch().localPoints(), true);

    vector ctr = 0.5*(bb.max() + bb.min());

    const vectorField& c = patch().Cf();

    scalarField coord = 2*((c - ctr) & y_)/((bb.max() - bb.min()) & y_);

    fixedValueFvPatchVectorField::operator==
    (
        n_*maxValue_*(1.0 - sqr(coord))
    );


    fixedValueFvPatchVectorField::updateCoeffs();


//Some outputs 
    word write_outputs = "no";		//yes or no  -  OpenFOAM equivalent to to char write_outputs[] = "yes";

    if (write_outputs == "yes")
	{
		Info << endl << "Face centers (c):" << endl;
		Info << c << endl; 

		Info << endl << "Patch center (ctr):" << endl;
		Info << ctr << endl; 

		Info << endl << "Patch (c - ctr):" << endl;
		Info << c - ctr << endl;

		Info << endl << "Patch ((c - ctr) & y_):" << endl;
		Info << ((c - ctr) & y_) << endl;

		Info << endl << "Direction vector y_" << endl;
		Info << y_ << endl;
	 
		Info << endl << "Patch max bound (bb.max):" << endl;
		Info <<  bb.max() << endl;

		Info << endl << "Patch min bound (bb.max):" << endl;
		Info << bb.min() << endl;

		Info << endl << "Patch ( (bb.max() - bb.min()) ):" << endl;
		Info << ((bb.max() - bb.min())) << endl;

		Info << endl << "Patch ( (bb.max() - bb.min()) & y_ ):" << endl;
		Info << ((bb.max() - bb.min()) & y_) << endl;

		Info << endl << "Patch coord ( 2*((c - ctr) & y_) ):" << endl;
		Info << 2*((c - ctr) & y_) << endl;

		Info << endl << "Patch coord ( 2*((c - ctr) & y_)/((bb.max() - bb.min()) & y_) ):" << endl;
		Info << coord << endl;

		Info << endl << "Patch ( 1.0 - sqr(coord)) :" << endl;
		Info << n_*maxValue_*(1.0 - sqr(coord))<< endl;

		Info << endl << "Patch operator == n_*maxValue_*(1.0 - sqr(coord)) :" << endl;
		Info << n_*maxValue_*(1.0 - sqr(coord))<< endl;

		vectorField v_test = n_*maxValue_*(1.0 - sqr(coord));

		Info << endl << "Loop for c, operator, operator.x()" << endl;
		forAll(c, faceI)
		{
		    Info << c[faceI] << "     " << n_*maxValue_*(1.0 - sqr(coord[faceI])) << "     " << v_test[faceI][0] << endl;
		}

		Info << endl << endl;
	}
}


void Foam::myParabolicVelocityFvPatchVectorField::write
(
    Ostream& os
) const
{
    fvPatchVectorField::write(os);
    os.writeKeyword("maxValue") << maxValue_ << token::END_STATEMENT << nl;
    os.writeKeyword("n") << n_ << token::END_STATEMENT << nl;
    os.writeKeyword("y") << y_ << token::END_STATEMENT << nl;
}


// * * * * * * * * * * * * * * Build Macro Function  * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
        fvPatchVectorField,
        myParabolicVelocityFvPatchVectorField
    );
}

// ************************************************************************* //
