/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2018 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "myParabolicVelocityModFvPatchVectorField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"

// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

Foam::scalar Foam::myParabolicVelocityModFvPatchVectorField::t() const
{
    return db().time().timeOutputValue();
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::myParabolicVelocityModFvPatchVectorField::
myParabolicVelocityModFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchVectorField(p, iF),
    maxValue_(0.0),
    n_(Zero),
    y_(Zero),
    useTime_("no"),
    printExtraInfo_("no")
{
}


Foam::myParabolicVelocityModFvPatchVectorField::
myParabolicVelocityModFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchVectorField(p, iF),
    maxValue_(readScalar(dict.lookup("maxValue"))),
    n_(pTraits<vector>(dict.lookup("n"))),
    y_(pTraits<vector>(dict.lookup("y"))),
    useTime_(dict.lookupOrDefault<word>("useTime", "no")),
    printExtraInfo_(dict.lookupOrDefault<word>("printExtraInfo", "no"))
{

    if (mag(n_) < SMALL || mag(y_) < SMALL)
    {
         FatalErrorIn("myParabolicVelocityFvPatchVectorField(dict)")
             << "n or y given with zero size not correct"
             << abort(FatalError);
    }
 
    n_ /= mag(n_);
    y_ /= mag(y_);

    fixedValueFvPatchVectorField::evaluate();

    /*
    //Initialise with the value entry if evaluation is not possible
    fvPatchVectorField::operator=
    (
        vectorField("value", dict, p.size())
    );
    */
}


Foam::myParabolicVelocityModFvPatchVectorField::
myParabolicVelocityModFvPatchVectorField
(
    const myParabolicVelocityModFvPatchVectorField& ptf,
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchVectorField(ptf, p, iF, mapper),
    maxValue_(ptf.maxValue_),
    n_(ptf.n_),
    y_(ptf.y_),
    useTime_(ptf.useTime_),
    printExtraInfo_(ptf.printExtraInfo_)
{}


Foam::myParabolicVelocityModFvPatchVectorField::
myParabolicVelocityModFvPatchVectorField
(
    const myParabolicVelocityModFvPatchVectorField& ptf
)
:
    fixedValueFvPatchVectorField(ptf),
    maxValue_(ptf.maxValue_),
    n_(ptf.n_),
    y_(ptf.y_),
    useTime_(ptf.useTime_),
    printExtraInfo_(ptf.printExtraInfo_)
{}


Foam::myParabolicVelocityModFvPatchVectorField::
myParabolicVelocityModFvPatchVectorField
(
    const myParabolicVelocityModFvPatchVectorField& ptf,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchVectorField(ptf, iF),
    maxValue_(ptf.maxValue_),
    n_(ptf.n_),
    y_(ptf.y_),
    useTime_(ptf.useTime_),
    printExtraInfo_(ptf.printExtraInfo_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::myParabolicVelocityModFvPatchVectorField::autoMap
(
    const fvPatchFieldMapper& m
)
{
    fixedValueFvPatchVectorField::autoMap(m);

}


void Foam::myParabolicVelocityModFvPatchVectorField::rmap
(
    const fvPatchVectorField& ptf,
    const labelList& addr
)
{
    fixedValueFvPatchVectorField::rmap(ptf, addr);

    const myParabolicVelocityModFvPatchVectorField& tiptf =
        refCast<const myParabolicVelocityModFvPatchVectorField>(ptf);


}


void Foam::myParabolicVelocityModFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }

    boundBox bb(patch().patch().localPoints(), true);

    vector ctr = 0.5*(bb.max() + bb.min());

    const vectorField& c = patch().Cf();

    scalarField coord = 2*((c - ctr) & y_)/((bb.max() - bb.min()) & y_);


    if (useTime_=="yes")
	{
    	fixedValueFvPatchVectorField::operator==
    	(
        	std::sin(t())*n_*maxValue_*(1.0 - sqr(coord))
    	);
	}
    else 
	{
    	fixedValueFvPatchVectorField::operator==
    	(
        	n_*maxValue_*(1.0 - sqr(coord))
    	);
	}


    fixedValueFvPatchVectorField::updateCoeffs();



//Some outputs 

    //word write_outputs = "no";		//yes or no  -  OpenFOAM equivalent to to char write_outputs[] = "yes";
    //if (write_outputs == "yes")

    if (printExtraInfo_=="yes")
	{
		Info << endl << "Flag useTime is set to:" << endl;
		Info << useTime_ << endl;

		Info << endl << "Flag printExtraInfo is set to:" << endl;
		Info << printExtraInfo_ << endl;

		Info << endl << "time:" << endl;
		Info << t() << endl;


		Info << endl << "********** Begin Patch info" << endl;

		Info << endl << "Output of patch().name()" << endl;
        Info << patch().name() << endl << endl;

		Info << endl << "Output of patch().patch()" << endl;
        Info << patch().patch() << endl << endl;

		Info << endl << "Output of patch().patch().localPoints()" << endl;
        Info << patch().patch().localPoints() << endl;

		Info << endl << "********** End Patch info" << endl;


		Info << endl << "Face centers (c):" << endl;
		Info << c << endl; 

		Info << endl << "Patch center (ctr):" << endl;
		Info << ctr << endl; 

		Info << endl << "Patch (c - ctr):" << endl;
		Info << c - ctr << endl;

		Info << endl << "Patch ((c - ctr) & y_):" << endl;
		Info << ((c - ctr) & y_) << endl;

		Info << endl << "Direction vector y_" << endl;
		Info << y_ << endl;
	 
		Info << endl << "Patch max bound (bb.max):" << endl;
		Info <<  bb.max() << endl;

		Info << endl << "Patch min bound (bb.max):" << endl;
		Info << bb.min() << endl;

		Info << endl << "Patch ( (bb.max() - bb.min()) ):" << endl;
		Info << ((bb.max() - bb.min())) << endl;

		Info << endl << "Patch ( (bb.max() - bb.min()) & y_ ):" << endl;
		Info << ((bb.max() - bb.min()) & y_) << endl;

		Info << endl << "Patch coord ( 2*((c - ctr) & y_) ):" << endl;
		Info << 2*((c - ctr) & y_) << endl;

		Info << endl << "Patch coord ( 2*((c - ctr) & y_)/((bb.max() - bb.min()) & y_) ):" << endl;
		Info << coord << endl;

		Info << endl << "Patch ( 1.0 - sqr(coord)) :" << endl;
		Info << n_*maxValue_*(1.0 - sqr(coord))<< endl;

		Info << endl << "Patch operator == n_*maxValue_*(1.0 - sqr(coord)) :" << endl;
		Info << n_*maxValue_*(1.0 - sqr(coord))<< endl;

		vectorField v_test = n_*maxValue_*(1.0 - sqr(coord));

		Info << endl << "Loop for c, operator, operator.x()" << endl;
		forAll(c, faceI)
		{
		    Info << c[faceI] << "     " << n_*maxValue_*(1.0 - sqr(coord[faceI])) << "     " << v_test[faceI][0] << endl;
		}

		Info << endl << endl;
	}

}


//These lines are used to write out the output dictionaries
//If you use the line with scalarValue, it will write the word scalarValue 
//in the dictionary U with the value maxValue_
void Foam::myParabolicVelocityModFvPatchVectorField::write
(
    Ostream& os
) const
{
    fvPatchVectorField::write(os);
    //os.writeKeyword("scalarValue") << maxValue_ << token::END_STATEMENT << nl;
    os.writeKeyword("maxValue") << maxValue_ << token::END_STATEMENT << nl;
    os.writeKeyword("n") << n_ << token::END_STATEMENT << nl;
    os.writeKeyword("y") << y_ << token::END_STATEMENT << nl;
    os.writeKeyword("useTime") << useTime_ << token::END_STATEMENT << nl;
    os.writeKeyword("printExtraInfo") << printExtraInfo_ << token::END_STATEMENT << nl;
}


// * * * * * * * * * * * * * * Build Macro Function  * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
        fvPatchVectorField,
        myParabolicVelocityModFvPatchVectorField
    );
}

// ************************************************************************* //
